searchData={"items":[{"type":"module","title":"Wallaby","doc":"A concurrent feature testing library.","ref":"Wallaby.html"},{"type":"module","title":"Configuration - Wallaby","doc":"Wallaby supports the following options:\n\n* `:otp_app` - The name of your OTP application. This is used to check out your Ecto repos into the SQL Sandbox.\n* `:screenshot_dir` - The directory to store screenshots.\n* `:screenshot_on_failure` - if Wallaby should take screenshots on test failures (defaults to `false`).\n* `:max_wait_time` - The amount of time that Wallaby should wait to find an element on the page. (defaults to `3_000`)\n* `:js_errors` - if Wallaby should re-throw JavaScript errors in elixir (defaults to true).\n* `:js_logger` - IO device where JavaScript console logs are written to. Defaults to :stdio. This option can also be set to a file or any other io device. You can disable JavaScript console logging by setting this to `nil`.","ref":"Wallaby.html#module-configuration"},{"type":"function","title":"Wallaby.driver/0","doc":"","ref":"Wallaby.html#driver/0"},{"type":"function","title":"Wallaby.end_session/1","doc":"Ends a browser session.","ref":"Wallaby.html#end_session/1"},{"type":"function","title":"Wallaby.start_session/1","doc":"Starts a browser session.","ref":"Wallaby.html#start_session/1"},{"type":"function","title":"Multiple sessions - Wallaby.start_session/1","doc":"Each session runs in its own browser so that each test runs in isolation.\nBecause of this isolation multiple sessions can be created for a test:\n\n```\n@message_field Query.text_field(\"Share Message\")\n@share_button Query.button(\"Share\")\n@message_list Query.css(\".messages\")\n\ntest \"That multiple sessions work\" do\n  {:ok, user1} = Wallaby.start_session\n  user1\n  |> visit(\"/page.html\")\n  |> fill_in(@message_field, with: \"Hello there!\")\n  |> click(@share_button)\n\n  {:ok, user2} = Wallaby.start_session\n  user2\n  |> visit(\"/page.html\")\n  |> fill_in(@message_field, with: \"Hello yourself\")\n  |> click(@share_button)\n\n  assert user1 |> find(@message_list) |> List.last |> text == \"Hello yourself\"\n  assert user2 |> find(@message_list) |> List.first |> text == \"Hello there\"\nend\n```","ref":"Wallaby.html#start_session/1-multiple-sessions"},{"type":"type","title":"Wallaby.reason/0","doc":"","ref":"Wallaby.html#t:reason/0"},{"type":"type","title":"Wallaby.start_session_opts/0","doc":"","ref":"Wallaby.html#t:start_session_opts/0"},{"type":"module","title":"Wallaby.Browser","doc":"The Browser module is the entrypoint for interacting with a real browser.\n\nBy default, action only work with elements that are visible to a real user.","ref":"Wallaby.Browser.html"},{"type":"module","title":"Actions - Wallaby.Browser","doc":"Actions are used to interact with form elements. All actions work with the\nquery interface:\n\n```html\n \n  First Name\n \n \n```\n\n```\nfill_in(page, Query.text_field(\"First Name\"), with: \"Grace\")\nfill_in(page, Query.text_field(\"first_name\"), with: \"Grace\")\nfill_in(page, Query.text_field(\"user_first_name\"), with: \"Grace\")\n```\n\nThese queries work with any of the available actions.\n\n```\nfill_in(page, Query.text_field(\"First Name\"), with: \"Chris\")\nclear(page, Query.text_field(\"user_email\"))\nclick(page, Query.radio_button(\"Radio Button 1\"))\nclick(page, Query.checkbox(\"Checkbox\"))\nclick(page, Query.checkbox(\"Checkbox\"))\nclick(page, Query.option(\"Option 1\"))\nclick(page, Query.button(\"Some Button\"))\nattach_file(page, Query.file_field(\"Avatar\"), path: \"test/fixtures/avatar.jpg\")\n```\n\nActions return their parent element so that they can be chained together:\n\n```\npage\n|> find(Query.css(\".signup-form\"), fn(form) ->\n  form\n  |> fill_in(Query.text_field(\"Name\"), with: \"Grace Hopper\")\n  |> fill_in(Query.text_field(\"Email\"), with: \"grace@hopper.com\")\n  |> click(Query.button(\"Submit\"))\nend)\n```","ref":"Wallaby.Browser.html#module-actions"},{"type":"module","title":"Scoping - Wallaby.Browser","doc":"Finders provide scoping like so:\n\n```\nsession\n|> visit(\"/page.html\")\n|> find(Query.css(\".users\"))\n|> find(Query.css(\".user\", count: 3))\n|> List.first\n|> find(Query.css(\".user-name\"))\n```\n\nIf a callback is passed to find then the scoping will only apply to the callback\nand the parent will be passed to the next action in the chain:\n\n```\npage\n|> find(Query.css(\".todo-form\"), fn(form) ->\n  form\n  |> fill_in(Query.text_field(\"What needs doing?\"), with: \"Write Wallaby Documentation\")\n  |> click(Query.button(\"Save\"))\nend)\n|> find(Query.css(\".success-notification\"), fn(notification) ->\n  assert notification\n  |> has_text?(\"Todo created successfully!\")\nend)\n```\n\nThis allows you to create a test that is logically grouped together in a single pipeline.\nIt also means that its easy to create re-usable helper functions without having to worry about\nchaining. You could re-write the above example like this:\n\n```\ndef create_todo(page, todo) do\n  find(Query.css(\".todo-form\"), & fill_in_and_save_todo(&1, todo))\nend\n\ndef fill_in_and_save_todo(form, todo) do\n  form\n  |> fill_in(Query.text_field(\"What needs doing?\"), with: todo)\n  |> click(Query.button(\"Save\"))\nend\n\ndef todo_was_created?(page) do\n  find Query.css(page, \".success-notification\"), fn(notification) ->\n    assert notification\n    |> has_text?(\"Todo created successfully!\")\n  end\nend\n\nassert page\n|> create_todo(\"Write Wallaby Documentation\")\n|> todo_was_created?\n```","ref":"Wallaby.Browser.html#module-scoping"},{"type":"function","title":"Wallaby.Browser.accept_alert/2","doc":"Accepts one alert dialog, which must be triggered within the specified `fun`.\nReturns the message that was presented to the user. For example:\n\n```\nmessage = accept_alert session, fn(s) ->\n  click(s, Query.link(\"Trigger alert\"))\nend\n```","ref":"Wallaby.Browser.html#accept_alert/2"},{"type":"function","title":"Wallaby.Browser.accept_confirm/2","doc":"Accepts one confirmation dialog, which must be triggered within the specified\n`fun`. Returns the message that was presented to the user. For example:\n\n```\nmessage = accept_confirm session, fn(s) ->\n  click(s, Query.link(\"Trigger confirm\"))\nend\n```","ref":"Wallaby.Browser.html#accept_confirm/2"},{"type":"function","title":"Wallaby.Browser.accept_prompt/2","doc":"Accepts one prompt, which must be triggered within the specified `fun`. The\n`[with: value]` option allows to simulate user input for the prompt. If no\nvalue is provided, the default value that was passed to `window.prompt` will\nbe used instead. Returns the message that was presented to the user. For\nexample:\n\n```\nmessage = accept_prompt session, fn(s) ->\n  click(s, Query.link(\"Trigger prompt\"))\nend\n```\n\nExample providing user input:\n\n```\nmessage = accept_prompt session, [with: \"User input\"], fn(s) ->\n  click(s, Query.link(\"Trigger prompt\"))\nend\n```","ref":"Wallaby.Browser.html#accept_prompt/2"},{"type":"function","title":"Wallaby.Browser.accept_prompt/3","doc":"","ref":"Wallaby.Browser.html#accept_prompt/3"},{"type":"function","title":"Wallaby.Browser.all/2","doc":"Finds all of the DOM elements that match the CSS selector. If no elements are\nfound then an empty list is immediately returned. This is equivalent to calling\n`find(session, css(\"element\", count: nil, minimum: 0))`.","ref":"Wallaby.Browser.html#all/2"},{"type":"macro","title":"Wallaby.Browser.assert_has/2","doc":"Checks if `query` is present within `parent` and raises if not found.\n\nReturns the given `parent` if the assertion is correct so that it is easily\npipeable.","ref":"Wallaby.Browser.html#assert_has/2"},{"type":"macro","title":"Examples - Wallaby.Browser.assert_has/2","doc":"session\n    |> visit(\"/\")\n    |> assert_has(Query.css(\".login-button\"))","ref":"Wallaby.Browser.html#assert_has/2-examples"},{"type":"function","title":"Wallaby.Browser.assert_text/2","doc":"","ref":"Wallaby.Browser.html#assert_text/2"},{"type":"function","title":"Wallaby.Browser.assert_text/3","doc":"Matches the Element's content with the provided text and raises if not found.\n\nReturns the given `parent` if the assertion is correct so that it is easily\npipeable.","ref":"Wallaby.Browser.html#assert_text/3"},{"type":"function","title":"Examples - Wallaby.Browser.assert_text/3","doc":"```\nsession\n|> visit(\"/\")\n|> assert_text(\"Login\")\n```\n\nExample providing query:\n\n```\nsession\n|> visit(\"/\")\n|> assert_text(Query.css(\".login-button\"), \"Login\")\n```","ref":"Wallaby.Browser.html#assert_text/3-examples"},{"type":"function","title":"Wallaby.Browser.attach_file/3","doc":"Attaches a file to a file input. Input elements are looked up by id, label text,\nor name.\n\nWhen dealing with Selenium server (especially a remote instance), the file\nmust be uploaded to Selenium via `send_keys` recognizing a local file,\nelse attaching a file fails.","ref":"Wallaby.Browser.html#attach_file/3"},{"type":"function","title":"Wallaby.Browser.attr/3","doc":"Gets the value of the elements attribute.","ref":"Wallaby.Browser.html#attr/3"},{"type":"function","title":"Wallaby.Browser.button_down/2","doc":"Clicks and holds the given mouse button at the current mouse coordinates.","ref":"Wallaby.Browser.html#button_down/2"},{"type":"function","title":"Wallaby.Browser.button_up/2","doc":"Releases given previously held mouse button.","ref":"Wallaby.Browser.html#button_up/2"},{"type":"function","title":"Wallaby.Browser.clear/2","doc":"","ref":"Wallaby.Browser.html#clear/2"},{"type":"function","title":"Wallaby.Browser.click/2","doc":"Clicks the mouse on the element returned by the query or at the current mouse cursor position.","ref":"Wallaby.Browser.html#click/2"},{"type":"function","title":"Wallaby.Browser.close_window/1","doc":"Closes the current window.\n\nThe window is either an instance of a browser tab or another operating system window.","ref":"Wallaby.Browser.html#close_window/1"},{"type":"function","title":"Usage - Wallaby.Browser.close_window/1","doc":"```elixir\nfeature \"closing a window focuses the previously focused window\", %{session: session} do\n  original_handle =\n    session\n    |> visit(\"/home\")\n    |> window_handle()\n\n  new_handle =\n    session\n    # click a link that takes you to a new tab\n    |> click(Query.link(\"External Page\"))\n    |> close_window()\n    |> window_handle()\n\n  assert original_handle == new_handle\nend\n```","ref":"Wallaby.Browser.html#close_window/1-usage"},{"type":"function","title":"Wallaby.Browser.cookies/1","doc":"","ref":"Wallaby.Browser.html#cookies/1"},{"type":"function","title":"Wallaby.Browser.current_path/1","doc":"Gets the current path of the session","ref":"Wallaby.Browser.html#current_path/1"},{"type":"function","title":"Wallaby.Browser.current_url/1","doc":"Gets the current url of the session","ref":"Wallaby.Browser.html#current_url/1"},{"type":"function","title":"Wallaby.Browser.dismiss_confirm/2","doc":"Dismisses one confirmation dialog, which must be triggered within the\nspecified `fun`. Returns the message that was presented to the user. For\nexample:\n\n```\nmessage = dismiss_confirm session, fn(s) ->\n  click(s, Query.link(\"Trigger confirm\"))\nend\n```","ref":"Wallaby.Browser.html#dismiss_confirm/2"},{"type":"function","title":"Wallaby.Browser.dismiss_prompt/2","doc":"Dismisses one prompt, which must be triggered within the specified `fun`.\nReturns the message that was presented to the user. For example:\n\n```\nmessage = dismiss_prompt session, fn(s) ->\n  click(s, Query.link(\"Trigger prompt\"))\nend\n```","ref":"Wallaby.Browser.html#dismiss_prompt/2"},{"type":"function","title":"Wallaby.Browser.double_click/1","doc":"Double-clicks left mouse button at the current mouse coordinates.","ref":"Wallaby.Browser.html#double_click/1"},{"type":"function","title":"Wallaby.Browser.execute_query/2","doc":"","ref":"Wallaby.Browser.html#execute_query/2"},{"type":"function","title":"Wallaby.Browser.execute_script/2","doc":"Executes JavaScript synchronously, taking as arguments the script to execute,\nan optional list of arguments available in the script via `arguments`, and an\noptional callback function with the result of script execution as a parameter.","ref":"Wallaby.Browser.html#execute_script/2"},{"type":"function","title":"Wallaby.Browser.execute_script/3","doc":"","ref":"Wallaby.Browser.html#execute_script/3"},{"type":"function","title":"Wallaby.Browser.execute_script/4","doc":"","ref":"Wallaby.Browser.html#execute_script/4"},{"type":"function","title":"Wallaby.Browser.execute_script_async/2","doc":"Executes asynchronous JavaScript, taking as arguments the script to execute,\nan optional list of arguments available in the script via `arguments`, and an\noptional callback function with the result of script execution as a parameter.","ref":"Wallaby.Browser.html#execute_script_async/2"},{"type":"function","title":"Wallaby.Browser.execute_script_async/3","doc":"","ref":"Wallaby.Browser.html#execute_script_async/3"},{"type":"function","title":"Wallaby.Browser.execute_script_async/4","doc":"","ref":"Wallaby.Browser.html#execute_script_async/4"},{"type":"function","title":"Wallaby.Browser.fill_in/3","doc":"Fills in an element identified by `query` with `value`.\n\nAll inputs previously present in the input field will be overridden.","ref":"Wallaby.Browser.html#fill_in/3"},{"type":"function","title":"Examples - Wallaby.Browser.fill_in/3","doc":"page\n    |> fill_in(Query.text_field(\"name\"), with: \"Chris\")\n    |> fill_in(Query.css(\"#password_field\"), with: \"secret42\")","ref":"Wallaby.Browser.html#fill_in/3-examples"},{"type":"function","title":"Note - Wallaby.Browser.fill_in/3","doc":"Currently, ChromeDriver only supports [BMP Unicode](http://www.unicode.org/roadmaps/bmp/) characters. Emojis are [SMP](https://www.unicode.org/roadmaps/smp/) characters and will be ignored by ChromeDriver.\n\nUsing JavaScript is a known workaround for filling in fields with Emojis and other non-BMP characters.","ref":"Wallaby.Browser.html#fill_in/3-note"},{"type":"function","title":"Wallaby.Browser.find/2","doc":"Finds and returns one or more DOM element(s) on the page based on the given query.\n\nThe query is scoped by the first argument, which is either the `%Session{}` or an\n`%Element{}`.","ref":"Wallaby.Browser.html#find/2"},{"type":"function","title":"Example - Wallaby.Browser.find/2","doc":"```elixir\nsession\n|> find(Query.css(\"#login-button\"))\n|> assert_text(\"Login\")\n\nbuttons =\n  session\n  |> find(Query.css(\".login-button\", count: 2, text: \"Login\"))\n```","ref":"Wallaby.Browser.html#find/2-example"},{"type":"function","title":"Notes - Wallaby.Browser.find/2","doc":"- Blocks until it finds the element(s) or the max time is reached.\n- By default only 1 element is expected to match the query. If more elements are present then a count can be\n  specified. Use `count: :any` to allow any number of elements to be present.\n- By default only elements that would be visible to a real user on the page are returned.","ref":"Wallaby.Browser.html#find/2-notes"},{"type":"function","title":"Wallaby.Browser.find/3","doc":"Same as `find/2`, but takes a callback to enact side effects on the found element(s).","ref":"Wallaby.Browser.html#find/3"},{"type":"function","title":"Example - Wallaby.Browser.find/3","doc":"```elixir\nsession\n|> find(Query.css(\"#login-button\"), fn button ->\n  assert_text(button, \"Login\")\nend)\n\nsession\n|> find(Query.css(\".login-button\", count: 2, text: \"Login\"), fn buttons ->\n  assert 2 == length(buttons)\nend)\n\n```","ref":"Wallaby.Browser.html#find/3-example"},{"type":"function","title":"Notes - Wallaby.Browser.find/3","doc":"- Returns the first argument to make the function pipe-able.","ref":"Wallaby.Browser.html#find/3-notes"},{"type":"function","title":"Wallaby.Browser.focus_default_frame/1","doc":"Changes the driver focus to the default (top level) frame.","ref":"Wallaby.Browser.html#focus_default_frame/1"},{"type":"function","title":"Wallaby.Browser.focus_frame/2","doc":"Changes the driver focus to the frame found by query.","ref":"Wallaby.Browser.html#focus_frame/2"},{"type":"function","title":"Wallaby.Browser.focus_parent_frame/1","doc":"Changes the driver focus to the parent frame.","ref":"Wallaby.Browser.html#focus_parent_frame/1"},{"type":"function","title":"Wallaby.Browser.focus_window/2","doc":"Focuses the window identified by the given handle.\n\nThe window is either an instance of a browser tab or another operating system window.","ref":"Wallaby.Browser.html#focus_window/2"},{"type":"function","title":"Usage - Wallaby.Browser.focus_window/2","doc":"```elixir\nfeature \"can switch between different tabs\", %{session: session} do\n  handle =\n    session\n    |> visit(\"/home\")\n    |> window_handle()\n\n  path =\n    session\n    # click a link that takes you to a new tab\n    |> click(Query.link(\"External Page\"))\n    |> assert_text(\"Some text\")\n    |> focus_window(handle)\n    |> current_path()\n\n  assert \"/home\" == path\nend\n```","ref":"Wallaby.Browser.html#focus_window/2-usage"},{"type":"function","title":"Wallaby.Browser.has?/2","doc":"Validates that the query returns a result. This can be used to define other\ntypes of matchers.","ref":"Wallaby.Browser.html#has?/2"},{"type":"function","title":"Wallaby.Browser.has_css?/2","doc":"","ref":"Wallaby.Browser.html#has_css?/2"},{"type":"function","title":"Wallaby.Browser.has_css?/3","doc":"Searches for CSS on the page.","ref":"Wallaby.Browser.html#has_css?/3"},{"type":"function","title":"Wallaby.Browser.has_no_css?/2","doc":"","ref":"Wallaby.Browser.html#has_no_css?/2"},{"type":"function","title":"Wallaby.Browser.has_no_css?/3","doc":"Searches for CSS that should not be on the page","ref":"Wallaby.Browser.html#has_no_css?/3"},{"type":"function","title":"Wallaby.Browser.has_text?/2","doc":"","ref":"Wallaby.Browser.html#has_text?/2"},{"type":"function","title":"Wallaby.Browser.has_text?/3","doc":"Matches the parent's content with the provided text.\n\nReturns a boolean that indicates if the text was found.","ref":"Wallaby.Browser.html#has_text?/3"},{"type":"function","title":"Examples - Wallaby.Browser.has_text?/3","doc":"```\nsession\n|> visit(\"/\")\n|> has_text?(\"Login\")\n```\n\nExample providing query:\n\n```\nsession\n|> visit(\"/\")\n|> has_text?(Query.css(\".login-button\"), \"Login\")\n```","ref":"Wallaby.Browser.html#has_text?/3-examples"},{"type":"function","title":"Wallaby.Browser.has_value?/2","doc":"","ref":"Wallaby.Browser.html#has_value?/2"},{"type":"function","title":"Wallaby.Browser.has_value?/3","doc":"Matches the Element's value with the provided value.","ref":"Wallaby.Browser.html#has_value?/3"},{"type":"function","title":"Wallaby.Browser.hover/2","doc":"Hovers over an element.","ref":"Wallaby.Browser.html#hover/2"},{"type":"function","title":"Wallaby.Browser.maximize_window/1","doc":"Maximizes the current window.\n\nThe window is either an instance of a browser tab or another operating system window.\n\nFor most browsers, this requires a graphical window manager to be running.","ref":"Wallaby.Browser.html#maximize_window/1"},{"type":"function","title":"Usage - Wallaby.Browser.maximize_window/1","doc":"```elixir\nfeature \"maximizes the window to the full size of the display\", %{session: session} do\n  %{\"width\" => width, \"height\" => height} =\n    session\n    |> visit(\"/home\")\n    |> maximize_window()\n    |> window_size()\n\n  assert width == 1920\n  assert height == 1080\nend\n```","ref":"Wallaby.Browser.html#maximize_window/1-usage"},{"type":"function","title":"Wallaby.Browser.move_mouse_by/3","doc":"Moves mouse by an offset relative to current cursor position.","ref":"Wallaby.Browser.html#move_mouse_by/3"},{"type":"function","title":"Wallaby.Browser.move_window/3","doc":"Sets the position of the current window.\n\nThe window is either an instance of a browser tab or another operating system window.","ref":"Wallaby.Browser.html#move_window/3"},{"type":"function","title":"Usage - Wallaby.Browser.move_window/3","doc":"```elixir\nfeature \"gets the current display position of the window\", %{session: session} do\n  %{\"x\" => x, \"y\" => y} =\n    session\n    |> visit(\"/home\")\n    |> move_window(500, 500)\n    |> window_position()\n\n  assert x == 500\n  assert y == 500\nend\n```","ref":"Wallaby.Browser.html#move_window/3-usage"},{"type":"function","title":"Wallaby.Browser.page_source/1","doc":"Retrieves the source of the current page.","ref":"Wallaby.Browser.html#page_source/1"},{"type":"function","title":"Wallaby.Browser.page_title/1","doc":"Gets the title for the current page","ref":"Wallaby.Browser.html#page_title/1"},{"type":"macro","title":"Wallaby.Browser.refute_has/2","doc":"Checks if `query` is not present within `parent` and raises if it is found.\n\nReturns the given `parent` if the query is not found so that it is easily\npipeable.","ref":"Wallaby.Browser.html#refute_has/2"},{"type":"macro","title":"Examples - Wallaby.Browser.refute_has/2","doc":"session\n    |> visit(\"/\")\n    |> refute_has(Query.css(\".secret-admin-content\"))","ref":"Wallaby.Browser.html#refute_has/2-examples"},{"type":"function","title":"Wallaby.Browser.resize_window/3","doc":"Sets the size of the current window.\n\nThe window is either an instance of a browser tab or another operating system window.","ref":"Wallaby.Browser.html#resize_window/3"},{"type":"function","title":"Usage - Wallaby.Browser.resize_window/3","doc":"```elixir\nfeature \"sets the size of the window to mobile dimensions\", %{session: session} do\n  %{\"width\" => width, \"height\" => height} =\n    session\n    |> visit(\"/home\")\n    |> resize_window(375, 667)\n    |> window_size()\n\n  assert width == 375\n  assert height == 667\nend\n```","ref":"Wallaby.Browser.html#resize_window/3-usage"},{"type":"function","title":"Wallaby.Browser.retry/2","doc":"Attempts to synchronize with the browser. This is most often used to\nexecute queries repeatedly until it either exceeds the time limit or\nreturns a success.","ref":"Wallaby.Browser.html#retry/2"},{"type":"function","title":"Note - Wallaby.Browser.retry/2","doc":"It is possible that this function never halts. Whenever we experience a stale\nreference error we retry the query without checking to see if we've run over\nour time. In practice we should eventually be able to query the DOM in a stable\nstate. However, if this error does continue to occur it will cause wallaby to\nloop forever (or until the test is killed by exunit).","ref":"Wallaby.Browser.html#retry/2-note"},{"type":"function","title":"Wallaby.Browser.selected?/2","doc":"Checks if the element has been selected. Alias for checked?(element)","ref":"Wallaby.Browser.html#selected?/2"},{"type":"function","title":"Wallaby.Browser.send_keys/2","doc":"","ref":"Wallaby.Browser.html#send_keys/2"},{"type":"function","title":"Wallaby.Browser.send_keys/3","doc":"Sends a list of key strokes to active element. If strings are included\nthen they are sent as individual keys. Special keys should be provided as a\nlist of atoms, which are automatically converted into the corresponding key\ncodes.\n\nFor a list of available key codes see `Wallaby.Helpers.KeyCodes`.","ref":"Wallaby.Browser.html#send_keys/3"},{"type":"function","title":"Example - Wallaby.Browser.send_keys/3","doc":"iex> Wallaby.Browser.send_keys(session, [\"Example Text\", :enter])\n    iex> Wallaby.Browser.send_keys(session, [:enter])\n    iex> Wallaby.Browser.send_keys(session, [:shift, :enter])","ref":"Wallaby.Browser.html#send_keys/3-example"},{"type":"function","title":"Note - Wallaby.Browser.send_keys/3","doc":"Currently, ChromeDriver only supports [BMP Unicode](http://www.unicode.org/roadmaps/bmp/) characters. Emojis are [SMP](https://www.unicode.org/roadmaps/smp/) characters and will be ignored by ChromeDriver.\n\nUsing JavaScript is a known workaround for filling in fields with Emojis and other non-BMP characters.","ref":"Wallaby.Browser.html#send_keys/3-note"},{"type":"function","title":"Wallaby.Browser.set_cookie/4","doc":"","ref":"Wallaby.Browser.html#set_cookie/4"},{"type":"function","title":"Wallaby.Browser.set_value/3","doc":"Sets the value of an element. The allowed type for the value depends on the\ntype of the element. The value may be:\n* a string of characters for a text element\n* :selected for a radio button, checkbox or select list option\n* :unselected for a checkbox","ref":"Wallaby.Browser.html#set_value/3"},{"type":"function","title":"Wallaby.Browser.take_screenshot/2","doc":"Takes a screenshot of the current window.\nScreenshots are saved to a \"screenshots\" directory in the same directory the\ntests are run in.\n\nPass `[{:name, \"some_name\"}]` to specify the file name. Defaults to a timestamp.\nPass `[{:log, true}]` to log the location of the screenshot to stdout. Defaults to false.","ref":"Wallaby.Browser.html#take_screenshot/2"},{"type":"function","title":"Wallaby.Browser.tap/2","doc":"Taps the element.","ref":"Wallaby.Browser.html#tap/2"},{"type":"function","title":"Wallaby.Browser.text/1","doc":"","ref":"Wallaby.Browser.html#text/1"},{"type":"function","title":"Wallaby.Browser.text/2","doc":"Gets the Element's text value.\n\nIf the element is not visible, the return value will be `\"\"`.","ref":"Wallaby.Browser.html#text/2"},{"type":"function","title":"Wallaby.Browser.touch_down/3","doc":"Touches the screen at the given position.","ref":"Wallaby.Browser.html#touch_down/3"},{"type":"function","title":"Wallaby.Browser.touch_down/4","doc":"Touches and holds the element on its top-left corner plus an optional offset.","ref":"Wallaby.Browser.html#touch_down/4"},{"type":"function","title":"Wallaby.Browser.touch_move/3","doc":"Moves the touch pointer (finger, stylus etc.) on the screen to the point determined by the given coordinates.","ref":"Wallaby.Browser.html#touch_move/3"},{"type":"function","title":"Wallaby.Browser.touch_scroll/4","doc":"Scroll on the screen from the given element by the given offset using touch events.","ref":"Wallaby.Browser.html#touch_scroll/4"},{"type":"function","title":"Wallaby.Browser.touch_up/1","doc":"Stops touching the screen.","ref":"Wallaby.Browser.html#touch_up/1"},{"type":"function","title":"Wallaby.Browser.visible?/2","doc":"Checks if the element is visible on the page","ref":"Wallaby.Browser.html#visible?/2"},{"type":"function","title":"Wallaby.Browser.visit/2","doc":"Changes the current page to the provided route.\nRelative paths are appended to the provided base_url.\nAbsolute paths do not use the base_url.","ref":"Wallaby.Browser.html#visit/2"},{"type":"function","title":"Wallaby.Browser.window_handle/1","doc":"Gets the window handle of the current window.\n\nThe window is either an instance of a browser tab or another operating system window.\nGetting the current window handle makes it easy to return to the window in case you\nneed to switch between them.","ref":"Wallaby.Browser.html#window_handle/1"},{"type":"function","title":"Usage - Wallaby.Browser.window_handle/1","doc":"```elixir\nfeature \"can open a new tab and switch back to the original tab\", %{session: session} do\n  handle =\n    session\n    |> visit(\"/home\")\n    |> window_handle()\n\n  path =\n    session\n    # click a link that takes you to a new tab\n    |> click(Query.link(\"External Page\"))\n    |> assert_text(\"Some text\")\n    |> focus_window(handle)\n    |> current_path()\n\n  assert \"/home\" == path\nend\n```","ref":"Wallaby.Browser.html#window_handle/1-usage"},{"type":"function","title":"Wallaby.Browser.window_handles/1","doc":"Gets the window handles of all available windows.\n\nThe window is either an instance of a browser tab or another operating system window.","ref":"Wallaby.Browser.html#window_handles/1"},{"type":"function","title":"Usage - Wallaby.Browser.window_handles/1","doc":"```elixir\nfeature \"can open new tabs for external links\", %{session: session} do\n  handles =\n    session\n    |> visit(\"/home\")\n    |> click(Query.link(\"External Page\"))\n    |> click(Query.link(\"Another External Page\"))\n    |> window_handles()\n\n  assert 3 == length(path)\nend\n```","ref":"Wallaby.Browser.html#window_handles/1-usage"},{"type":"function","title":"Wallaby.Browser.window_position/1","doc":"Gets the position of the current window.\n\nThe window is either an instance of a browser tab or another operating system window.","ref":"Wallaby.Browser.html#window_position/1"},{"type":"function","title":"Usage - Wallaby.Browser.window_position/1","doc":"```elixir\nfeature \"gets the current display position of the window\", %{session: session} do\n  %{\"x\" => x, \"y\" => y} =\n    session\n    |> visit(\"/home\")\n    |> window_position()\n\n  assert x == 200\n  assert y == 200\nend\n```","ref":"Wallaby.Browser.html#window_position/1-usage"},{"type":"function","title":"Wallaby.Browser.window_size/1","doc":"Gets the size of the current window.\n\nThe window is either an instance of a browser tab or another operating system window.\n\nThis is useful for debugging responsive designs where the layout changes as the window size changes. The default window size is 1280x800.","ref":"Wallaby.Browser.html#window_size/1"},{"type":"function","title":"Usage - Wallaby.Browser.window_size/1","doc":"```elixir\nfeature \"gets the size of the current window\", %{session: session} do\n  %{\"width\" => width, \"height\" => height} =\n    session\n    |> visit(\"/home\")\n    |> window_size()\n\n  assert width == 1280\n  assert height == 800\nend\n```","ref":"Wallaby.Browser.html#window_size/1-usage"},{"type":"type","title":"Wallaby.Browser.opts/0","doc":"","ref":"Wallaby.Browser.html#t:opts/0"},{"type":"type","title":"Wallaby.Browser.parent/0","doc":"","ref":"Wallaby.Browser.html#t:parent/0"},{"type":"opaque","title":"Wallaby.Browser.queryable/0","doc":"","ref":"Wallaby.Browser.html#t:queryable/0"},{"type":"type","title":"Wallaby.Browser.sync_result/0","doc":"","ref":"Wallaby.Browser.html#t:sync_result/0"},{"type":"type","title":"Wallaby.Browser.t/0","doc":"","ref":"Wallaby.Browser.html#t:t/0"},{"type":"type","title":"Wallaby.Browser.take_screenshot_opt/0","doc":"","ref":"Wallaby.Browser.html#t:take_screenshot_opt/0"},{"type":"module","title":"Wallaby.Chrome","doc":"The Chrome driver uses [Chromedriver](https://chromedriver.chromium.org/) to power Google Chrome and Chromium.","ref":"Wallaby.Chrome.html"},{"type":"module","title":"Usage - Wallaby.Chrome","doc":"Start a Wallaby Session using this driver with the following command:\n\n```elixir\n{:ok, session} = Wallaby.start_session()\n```","ref":"Wallaby.Chrome.html#module-usage"},{"type":"module","title":"Configuration - Wallaby.Chrome","doc":"","ref":"Wallaby.Chrome.html#module-configuration"},{"type":"module","title":"Headless - Wallaby.Chrome","doc":"Chrome will run in headless mode by default.\nYou can disable this behaviour using the following configuration.\n\nThis will override the default capabilities and capabilities set with application configuration.\nThis will _not_ override capabilities passed in directly to `Wallaby.start_session/1`.\n\n```elixir\nconfig :wallaby,\n  chromedriver: [\n    headless: false\n  ]\n```","ref":"Wallaby.Chrome.html#module-headless"},{"type":"module","title":"Capabilities - Wallaby.Chrome","doc":"These capabilities will override the default capabilities.\n\n```elixir\nconfig :wallaby,\n  chromedriver: [\n    capabilities: %{\n      # something\n    }\n  ]\n```","ref":"Wallaby.Chrome.html#module-capabilities"},{"type":"module","title":"ChromeDriver binary - Wallaby.Chrome","doc":"If ChromeDriver is not available in your path, you can specify it's location.\n\n```elixir\nconfig :wallaby,\n  chromedriver: [\n    path: \"path/to/chrome\"\n  ]\n```","ref":"Wallaby.Chrome.html#module-chromedriver-binary"},{"type":"module","title":"Chrome binary - Wallaby.Chrome","doc":"This configures which instance of Google Chrome to use.\n\nThis will override the default capabilities and capabilities set with application configuration.\nThis will _not_ override capabilities passed in directly to `Wallaby.start_session/1`.\n\n```elixir\nconfig :wallaby,\n  chromedriver: [\n    binary: \"path/to/chrome\"\n  ]\n```","ref":"Wallaby.Chrome.html#module-chrome-binary"},{"type":"module","title":"Default Capabilities - Wallaby.Chrome","doc":"By default, Chromdriver will use the following capabilities\n\nYou can read more about capabilities in the [JSON Wire Protocol](https://github.com/SeleniumHQ/selenium/wiki/JsonWireProtocol#capabilities-json-object) documentation and the [Chromedriver](https://sites.google.com/a/chromium.org/chromedriver/capabilities) documentation.\n\n```elixir\n  %{\n    javascriptEnabled: false,\n    loadImages: false,\n    version: \"\",\n    rotatable: false,\n    takesScreenshot: true,\n    cssSelectorsEnabled: true,\n    nativeEvents: false,\n    platform: \"ANY\",\n    unhandledPromptBehavior: \"accept\",\n    loggingPrefs: %{\n      browser: \"DEBUG\"\n    },\n    chromeOptions: %{\n      args: [\n        \"--no-sandbox\",\n        \"window-size=1280,800\",\n        \"--disable-gpu\",\n        \"--headless\",\n        \"--fullscreen\",\n        \"--user-agent=Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36\"\n      ]\n    }\n  }\n```","ref":"Wallaby.Chrome.html#module-default-capabilities"},{"type":"module","title":"Notes - Wallaby.Chrome","doc":"This driver requires [Chromedriver](https://sites.google.com/a/chromium.org/chromedriver/) to be installed in your path.","ref":"Wallaby.Chrome.html#module-notes"},{"type":"function","title":"Wallaby.Chrome.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Wallaby.Chrome.html#child_spec/1"},{"type":"type","title":"Wallaby.Chrome.start_session_opts/0","doc":"Options to pass to Wallaby.start_session/1\n\n```elixir\nWallaby.start_session(\n  capabilities: %{chromeOptions: %{args: [\"--headless\"]}},\n  create_session_fn: fn url, capabilities ->\n    WebdriverClient.create_session(url, capabilities)\n  end\n)\n```\n\n* `:capabilities` - capabilities to pass to chromedriver on session startup\n* `create_session_fn` - Deprecated and to be removed\n* `:readiness_timeout` - milliseconds to wait for chromedriver server to be ready\n  before raising a timeout error. (Default: 10000)","ref":"Wallaby.Chrome.html#t:start_session_opts/0"},{"type":"module","title":"Wallaby.DSL","doc":"Sets up the Wallaby DSL in a module.\n\nAll functions in `Wallaby.Browser` are now accessible without a module name\nand `Wallaby.Browser`, `Wallaby.Element` and `Wallaby.Query` are all aliased.","ref":"Wallaby.DSL.html"},{"type":"module","title":"Example - Wallaby.DSL","doc":"```elixir\ndefmodule MyPage do\n  use Wallaby.DSL\n\n  @name_field Query.text_field(\"Name\")\n  @email_field Query.text_field(\"email\")\n  @save_button Query.button(\"Save\")\n\n  def register(session) do\n    session\n    |> visit(\"/registration.html\")\n    |> fill_in(@name_field, with: \"Chris\")\n    |> fill_in(@email_field, with: \"c@keathly.io\")\n    |> click(@save_button)\n  end\nend\n```","ref":"Wallaby.DSL.html#module-example"},{"type":"module","title":"Wallaby.Element","doc":"Defines an Element Struct and interactions with Elements.\n\nTypically these functions are used in conjunction with a `find`:\n\n```\npage\n|> find(Query.css(\".some-element\"), fn(element) -> Element.click(element) end)\n```\n\nThese functions can be used to create new actions specific to your application:\n\n```\ndef create_todo(todo_field, todo_text) do\n  todo_field\n  |> Element.click()\n  |> Element.fill_in(with: todo_text)\n  |> Element.send_keys([:enter])\nend\n```","ref":"Wallaby.Element.html"},{"type":"module","title":"Retrying - Wallaby.Element","doc":"Unlike `Browser` the actions in `Element` do not retry if the element becomes stale. Instead an exception will be raised.","ref":"Wallaby.Element.html#module-retrying"},{"type":"function","title":"Wallaby.Element.attr/2","doc":"Gets the value of the element's attribute.","ref":"Wallaby.Element.html#attr/2"},{"type":"function","title":"Wallaby.Element.clear/1","doc":"Clears any value set in the element.","ref":"Wallaby.Element.html#clear/1"},{"type":"function","title":"Wallaby.Element.click/2","doc":"Clicks the element.","ref":"Wallaby.Element.html#click/2"},{"type":"function","title":"Wallaby.Element.fill_in/2","doc":"Fills in the element with the specified value.","ref":"Wallaby.Element.html#fill_in/2"},{"type":"function","title":"Wallaby.Element.hover/1","doc":"Hovers on the element.","ref":"Wallaby.Element.html#hover/1"},{"type":"function","title":"Wallaby.Element.location/1","doc":"Returns a tuple `{x, y}` with the coordinates of the left-top corner of given element.","ref":"Wallaby.Element.html#location/1"},{"type":"function","title":"Wallaby.Element.selected?/1","doc":"Returns a boolean based on whether or not the element is selected.","ref":"Wallaby.Element.html#selected?/1"},{"type":"function","title":"Note - Wallaby.Element.selected?/1","doc":"This only really makes sense for options, checkboxes, and radio buttons.\nEverything else will simply return false because they have no notion of\n\"selected\".","ref":"Wallaby.Element.html#selected?/1-note"},{"type":"function","title":"Wallaby.Element.send_keys/2","doc":"Sends keys to the element.","ref":"Wallaby.Element.html#send_keys/2"},{"type":"function","title":"Wallaby.Element.set_value/2","doc":"Sets the value of the element.","ref":"Wallaby.Element.html#set_value/2"},{"type":"function","title":"Wallaby.Element.size/1","doc":"Returns a tuple `{width, height}` with the size of the given element.","ref":"Wallaby.Element.html#size/1"},{"type":"function","title":"Wallaby.Element.tap/1","doc":"Taps the element.","ref":"Wallaby.Element.html#tap/1"},{"type":"function","title":"Wallaby.Element.text/1","doc":"Gets the element's text value.\n\nIf the element is not visible, the return value will be `\"\"`.","ref":"Wallaby.Element.html#text/1"},{"type":"function","title":"Wallaby.Element.touch_down/3","doc":"Touches and holds the element on its top-left corner plus an optional offset.","ref":"Wallaby.Element.html#touch_down/3"},{"type":"function","title":"Wallaby.Element.touch_scroll/3","doc":"Scroll on the screen from the given element by the given offset using touch events.","ref":"Wallaby.Element.html#touch_scroll/3"},{"type":"function","title":"Wallaby.Element.value/1","doc":"Returns the Element's value.","ref":"Wallaby.Element.html#value/1"},{"type":"function","title":"Wallaby.Element.visible?/1","doc":"Returns a boolean based on whether or not the element is visible.","ref":"Wallaby.Element.html#visible?/1"},{"type":"type","title":"Wallaby.Element.attr/0","doc":"","ref":"Wallaby.Element.html#t:attr/0"},{"type":"type","title":"Wallaby.Element.keys_to_send/0","doc":"","ref":"Wallaby.Element.html#t:keys_to_send/0"},{"type":"type","title":"Wallaby.Element.t/0","doc":"","ref":"Wallaby.Element.html#t:t/0"},{"type":"type","title":"Wallaby.Element.value/0","doc":"","ref":"Wallaby.Element.html#t:value/0"},{"type":"module","title":"Wallaby.Feature","doc":"Helpers for writing features.\n\nYou can `use` or `import` this module.","ref":"Wallaby.Feature.html"},{"type":"module","title":"use Wallaby.Feature - Wallaby.Feature","doc":"Calling this module with `use` will automatically call `use Wallaby.DSL`.\n\nWhen called with `use` and you are using Ecto, please configure your `otp_app`.\n\n```\nconfig :wallaby, otp_app: :your_app\n```","ref":"Wallaby.Feature.html#module-use-wallaby-feature"},{"type":"macro","title":"Wallaby.Feature.feature/3","doc":"Defines a feature with a message.\n\nAdding `import Wallaby.Feature` to your test module will import the `Wallaby.Feature.feature/3` macro. This is a drop in replacement for the `ExUnit.Case.test/3` macro that you normally use.\n\nAdding `use Wallaby.Feature` to your test module will act the same as `import Wallaby.Feature`, as well as configure your Ecto repos properly and pass a `Wallaby.Session` into the test context.","ref":"Wallaby.Feature.html#feature/3"},{"type":"macro","title":"Sessions - Wallaby.Feature.feature/3","doc":"When called with `use`, the `Wallaby.Feature.feature/3` macro will automatically start a single session using the currently configured capabilities and is passed to the feature via the `:session` key in the context.\n\n```\nfeature \"test with a single session\", %{session: session} do\n  # ...\nend\n```\n\nIf you would like to start multiple sessions, assign the `@sessions` attribute to the number of sessions that the feature should start, and they will be pass to the feature via the `:sessions` key in the context.\n\n```\n@sessions 2\nfeature \"test with a two sessions\", %{sessions: [session_1, sessions_2]} do\n  # ...\nend\n```\n\nIf you need to change the headless mode, binary path, or capabilities sent to the session for a specific feature, you can assign `@sessions` to a list of keyword lists of the options to be passed to `Wallaby.start_session/1`. This will start the number of sessions equal to the size of the list.\n\n```\n@sessions [\n  [headless: false, binary: \"some_path\", capabilities: %{}]\n]\nfeature \"test with different capabilities\", %{session: session} do\n  # ...\nend\n```\n\nIf you don't wish to `use Wallaby.Feature` in your test module, you can add the following code to configure Ecto and create a session.\n\n```\nsetup tags do\n  :ok = Ecto.Adapters.SQL.Sandbox.checkout(YourApp.Repo)\n\n  unless tags[:async] do\n    Ecto.Adapters.SQL.Sandbox.mode(YourApp.Repo, {:shared, self()})\n  end\n\n  metadata = Phoenix.Ecto.SQL.Sandbox.metadata_for(YourApp.Repo, self())\n  {:ok, session} = Wallaby.start_session(metadata: metadata)\n\n  {:ok, session: session}\nend\n```","ref":"Wallaby.Feature.html#feature/3-sessions"},{"type":"macro","title":"Screenshots - Wallaby.Feature.feature/3","doc":"If you have configured `screenshot_on_failure` to be true, any exceptions raised during the feature will trigger a screenshot to be taken.","ref":"Wallaby.Feature.html#feature/3-screenshots"},{"type":"module","title":"Wallaby.Helpers.KeyCodes","doc":"Shortcuts for various keys.\n\n- :null\n- :cancel\n- :help\n- :backspace\n- :tab\n- :clear\n- :return\n- :enter\n- :shift\n- :control\n- :alt\n- :pause\n- :escape\n- :space\n- :pageup\n- :pagedown\n- :end\n- :home\n- :left_arrow\n- :up_arrow\n- :right_arrow\n- :down_arrow\n- :insert\n- :delete\n- :semicolon\n- :equals\n- :num0\n- :num1\n- :num2\n- :num3\n- :num4\n- :num5\n- :num6\n- :num7\n- :num8\n- :num9\n- :multiply\n- :add\n- :seperator\n- :subtract\n- :decimal\n- :divide\n- :command","ref":"Wallaby.Helpers.KeyCodes.html"},{"type":"function","title":"Wallaby.Helpers.KeyCodes.chars/1","doc":"","ref":"Wallaby.Helpers.KeyCodes.html#chars/1"},{"type":"function","title":"Wallaby.Helpers.KeyCodes.json/1","doc":"","ref":"Wallaby.Helpers.KeyCodes.html#json/1"},{"type":"module","title":"Wallaby.Query","doc":"Provides the query DSL.\n\nQueries are used to locate and retrieve DOM elements from a browser (see\n`Wallaby.Browser`). You create queries like so:\n\n```\nQuery.css(\".some-css\")\nQuery.xpath(\".//input\")\n```","ref":"Wallaby.Query.html"},{"type":"module","title":"Form elements - Wallaby.Query","doc":"There are several custom finders for locating form elements. Each of these allows\nfinding by their name, id text, or label text. This allows for more robust querying\nand decouples the query from presentation selectors like CSS classes.\n\n```\nQuery.text_field(\"My Name\")\nQuery.checkbox(\"Checkbox\")\nQuery.select(\"A Fancy Select Box\")\n```","ref":"Wallaby.Query.html#module-form-elements"},{"type":"module","title":"Query Options - Wallaby.Query","doc":"All of the query operations accept the following options:\n\n  - `:count` - The number of elements that should be found or `:any` (default: 1).\n      - If a `:minimum` or `:maximum` is specified, it defaults to `nil`.\n  - `:minimum` - The minimum number of elements that should be found, or `nil` (default: `nil`).\n  - `:maximum` - The maximum number of elements that should be found, or `nil` (default: `nil`).\n  - `:visible` - Determines if the query should return only visible elements (default: `true`).\n  - `:selected` - Determines if the query should return only selected elements (default: `:any`).\n  - `:text` - Text that should be found inside the element (default: `nil`).\n  - `:at` - The position (a number or `:all`) of the element to return if multiple elements satisfy the query. (default: `:all`)\n\nQuery options can also be set via functions by the same names:\n\n```\nQuery.css(\".names\")\n|> Query.visible(true)\n|> Query.count(3)\n```","ref":"Wallaby.Query.html#module-query-options"},{"type":"module","title":"Re-using queries - Wallaby.Query","doc":"It is often convenient to re-use queries. The easiest way is to use module\nattributes:\n\n```\n@name_field Query.text_field(\"User Name\")\n@submit_button Query.button(\"Save\")\n```\n\nIf the queries need to be dynamic then you should create a module that\nencapsulates the queries as functions:\n\n```\ndefmodule TodoListPage do\n  def todo_list do\n    Query.css(\".todo-list\")\n  end\n\n  def todos(count) do\n    Query.css(\".todo\", count: count)\n  end\nend\n```","ref":"Wallaby.Query.html#module-re-using-queries"},{"type":"module","title":"What does my query do? - Wallaby.Query","doc":"Wanna check out what exactly your query will do? Look no further than\n`Wallaby.Query.compile/1` - it takes a query and returns the CSS or xpath\nquery that will be sent to the driver:\n\n    iex> Wallaby.Query.compile Wallaby.Query.text(\"my text\")\n    {:xpath, \".//*[contains(normalize-space(text()), \\\"my text\\\")]\"}\n\nSo, whenever you're not sure whatever a specific query will do just compile\nit to get all the details!","ref":"Wallaby.Query.html#module-what-does-my-query-do"},{"type":"function","title":"Wallaby.Query.at/2","doc":"Updates a query's `at` option.","ref":"Wallaby.Query.html#at/2"},{"type":"function","title":"Example - Wallaby.Query.at/2","doc":"```\n  Query.css(\".names\")\n  |> Query.at(3)\n  ```","ref":"Wallaby.Query.html#at/2-example"},{"type":"function","title":"Wallaby.Query.at_number/1","doc":"","ref":"Wallaby.Query.html#at_number/1"},{"type":"function","title":"Wallaby.Query.attribute/3","doc":"Checks if the provided attribute, value pair is contained anywhere.","ref":"Wallaby.Query.html#attribute/3"},{"type":"function","title":"Wallaby.Query.button/2","doc":"Looks for a button (literal button or input type button, submit, image or\nreset) where the provided selector is the id, name, value, alt or title of the\nbutton.","ref":"Wallaby.Query.html#button/2"},{"type":"function","title":"Wallaby.Query.checkbox/2","doc":"Looks for a checkbox where the provided selector is the id, name or\nplaceholder of the checkbox itself or alternatively the id or the text of\nthe label.","ref":"Wallaby.Query.html#checkbox/2"},{"type":"function","title":"Wallaby.Query.compile/1","doc":"Compiles a query into CSS or xpath so its ready to be sent to the driver\n\n    iex> Wallaby.Query.compile Wallaby.Query.text(\"my text\")\n    {:xpath, \".//*[contains(normalize-space(text()), \\\"my text\\\")]\"}\n    iex> Wallaby.Query.compile Wallaby.Query.css(\"#some-id\")\n    {:css, \"#some-id\"}","ref":"Wallaby.Query.html#compile/1"},{"type":"function","title":"Wallaby.Query.count/1","doc":"","ref":"Wallaby.Query.html#count/1"},{"type":"function","title":"Wallaby.Query.count/2","doc":"Updates a query's `count`.\n\nThe `count` specifies how many elements you expect to be present within the scope\nof the query and can be any number greater than zero or `:any`.","ref":"Wallaby.Query.html#count/2"},{"type":"function","title":"Example - Wallaby.Query.count/2","doc":"```elixir\n# Exactly 2 elements\nQuery.css(\".names > li\")\n|> Query.count(2)\n\n# Any number of elements\nQuery.css(\".names > li\")\n|> Query.count(:any)\n```","ref":"Wallaby.Query.html#count/2-example"},{"type":"function","title":"Wallaby.Query.css/2","doc":"Literally queries for the CSS selector you provide.","ref":"Wallaby.Query.html#css/2"},{"type":"function","title":"Wallaby.Query.data/3","doc":"Checks if the data attribute is contained anywhere.","ref":"Wallaby.Query.html#data/3"},{"type":"function","title":"Wallaby.Query.file_field/2","doc":"Looks for a file input where the selector is the id or name of the file input\nitself or the id or text of the label.","ref":"Wallaby.Query.html#file_field/2"},{"type":"function","title":"Wallaby.Query.fillable_field/2","doc":"Looks for a text input field where the provided selector is the id, name or\nplaceholder of the text field itself or alternatively the id or the text of\nthe label.","ref":"Wallaby.Query.html#fillable_field/2"},{"type":"function","title":"Wallaby.Query.inner_text/1","doc":"","ref":"Wallaby.Query.html#inner_text/1"},{"type":"function","title":"Wallaby.Query.link/2","doc":"Looks for a link where the selector is the id, link text, title of the link\nitself or the alt of an image child node.","ref":"Wallaby.Query.html#link/2"},{"type":"function","title":"Wallaby.Query.matches_count?/2","doc":"","ref":"Wallaby.Query.html#matches_count?/2"},{"type":"function","title":"Wallaby.Query.option/2","doc":"Looks for an option that contains the given text.","ref":"Wallaby.Query.html#option/2"},{"type":"function","title":"Wallaby.Query.radio_button/2","doc":"Looks for a radio button where the provided selector is the id, name or\nplaceholder of the radio button itself or alternatively the id or the text of\nthe label.","ref":"Wallaby.Query.html#radio_button/2"},{"type":"function","title":"Wallaby.Query.result/1","doc":"","ref":"Wallaby.Query.html#result/1"},{"type":"function","title":"Wallaby.Query.select/2","doc":"Looks for a select box where the provided selector is the id or name of the\nselect box itself or alternatively the id or the text of the label.","ref":"Wallaby.Query.html#select/2"},{"type":"function","title":"Wallaby.Query.selected/2","doc":"Updates a query's `selected` option.","ref":"Wallaby.Query.html#selected/2"},{"type":"function","title":"Examples - Wallaby.Query.selected/2","doc":"```\n  Query.css(\"#select-dropdown\")\n  |> Query.selected(true)\n\n  Query.css(\"#select-dropdown\")\n  |> Query.selected(false)\n  ```","ref":"Wallaby.Query.html#selected/2-examples"},{"type":"function","title":"Wallaby.Query.selected?/1","doc":"","ref":"Wallaby.Query.html#selected?/1"},{"type":"function","title":"Wallaby.Query.specific_element_requested/1","doc":"","ref":"Wallaby.Query.html#specific_element_requested/1"},{"type":"function","title":"Wallaby.Query.text/2","doc":"This function can be used in one of two ways.\n\nThe first is by providing a selector and possible options. This generates a\nquery that checks if the provided text is contained anywhere.","ref":"Wallaby.Query.html#text/2"},{"type":"function","title":"Example - Wallaby.Query.text/2","doc":"```\n  Query.text(\"Submit\", count: 1)\n  ```\n\nThe second is by providing an existing query and a value to set as the `text`\noption.\n\nNote that the text you're querying for must appear in a single element.\nTo assert on text that appears in multiple (potentially nested) elements,\nrather than using `assert_has/2` in combination with `Query.text/2`, use\n`assert_text/{2,3}` directly.","ref":"Wallaby.Query.html#text/2-example"},{"type":"function","title":"Examples - Wallaby.Query.text/2","doc":"","ref":"Wallaby.Query.html#text/2-examples"},{"type":"function","title":"Querying the text of a specific element - Wallaby.Query.text/2","doc":"```\n  submit_button = Query.css(\"#submit-button\")\n\n  update_button = submit_button |> Query.text(\"Update\")\n  create_button = submit_button |> Query.text(\"Create\")\n  ```","ref":"Wallaby.Query.html#text/2-querying-the-text-of-a-specific-element"},{"type":"function","title":"Asserting on the text of a single element - Wallaby.Query.text/2","doc":"```\n    submit_button = Query.css(\"#submit-button\")\n    assert_has(session, Query.text(submit_button, \"Create\"))\n    ```","ref":"Wallaby.Query.html#text/2-asserting-on-the-text-of-a-single-element"},{"type":"function","title":"Asserting on the text of nested elements - Wallaby.Query.text/2","doc":"HTML:\n\n    ```\n     \n      Unread messages:\n       1 message \n     \n    ```\n\nTest:\n\n    ```\n    assert_text(session, \"Unread messages: 1 message\")\n\n    notifications_block = Query.css(\"#unread-notifications\")\n    # It would *not* work to query as: Query.text(notifications_block, \"Unread messages: 1 message\")\n    assert_has(session, Query.text(notifications_block, \"Unread messages:\"))\n    ```","ref":"Wallaby.Query.html#text/2-asserting-on-the-text-of-nested-elements"},{"type":"function","title":"Wallaby.Query.text_field/2","doc":"See `Wallaby.Query.fillable_field/2`.","ref":"Wallaby.Query.html#text_field/2"},{"type":"function","title":"Wallaby.Query.validate/1","doc":"","ref":"Wallaby.Query.html#validate/1"},{"type":"function","title":"Wallaby.Query.value/2","doc":"Checks if the provided value is contained anywhere.","ref":"Wallaby.Query.html#value/2"},{"type":"function","title":"Wallaby.Query.visible/2","doc":"Updates a query's visibility (visible if `true`, hidden if `false`).","ref":"Wallaby.Query.html#visible/2"},{"type":"function","title":"Examples - Wallaby.Query.visible/2","doc":"```\n  Query.css(\"#modal\")\n  |> Query.visible(true)\n\n  Query.css(\"#modal\")\n  |> Query.visible(false)\n  ```","ref":"Wallaby.Query.html#visible/2-examples"},{"type":"function","title":"Wallaby.Query.visible?/1","doc":"","ref":"Wallaby.Query.html#visible?/1"},{"type":"function","title":"Wallaby.Query.xpath/2","doc":"Literally queries for the xpath selector you provide.","ref":"Wallaby.Query.html#xpath/2"},{"type":"type","title":"Wallaby.Query.attribute_key_value_pair/0","doc":"","ref":"Wallaby.Query.html#t:attribute_key_value_pair/0"},{"type":"type","title":"Wallaby.Query.compiled/0","doc":"","ref":"Wallaby.Query.html#t:compiled/0"},{"type":"type","title":"Wallaby.Query.conditions/0","doc":"","ref":"Wallaby.Query.html#t:conditions/0"},{"type":"type","title":"Wallaby.Query.html_validation/0","doc":"","ref":"Wallaby.Query.html#t:html_validation/0"},{"type":"type","title":"Wallaby.Query.method/0","doc":"","ref":"Wallaby.Query.html#t:method/0"},{"type":"type","title":"Wallaby.Query.opts/0","doc":"","ref":"Wallaby.Query.html#t:opts/0"},{"type":"type","title":"Wallaby.Query.result/0","doc":"","ref":"Wallaby.Query.html#t:result/0"},{"type":"type","title":"Wallaby.Query.selector/0","doc":"","ref":"Wallaby.Query.html#t:selector/0"},{"type":"type","title":"Wallaby.Query.t/0","doc":"","ref":"Wallaby.Query.html#t:t/0"},{"type":"module","title":"Wallaby.Selenium","doc":"The Selenium driver uses [Selenium Server](https://github.com/SeleniumHQ/selenium) to power many types of browsers (Chrome, Firefox, Edge, etc).","ref":"Wallaby.Selenium.html"},{"type":"module","title":"Usage - Wallaby.Selenium","doc":"Start a Wallaby Session using this driver with the following command:\n\n```elixir\n{:ok, session} = Wallaby.start_session()\n```","ref":"Wallaby.Selenium.html#module-usage"},{"type":"module","title":"Configuration - Wallaby.Selenium","doc":"","ref":"Wallaby.Selenium.html#module-configuration"},{"type":"module","title":"Capabilities - Wallaby.Selenium","doc":"These capabilities will override the default capabilities.\n\n```elixir\nconfig :wallaby,\n  selenium: [\n    capabilities: %{\n      # something\n    }\n  ]\n```","ref":"Wallaby.Selenium.html#module-capabilities"},{"type":"module","title":"Selenium Remote URL - Wallaby.Selenium","doc":"It is possible to globally set Selenium's \"Remote URL\" by setting the following option.\n\nBy default it is http://localhost:4444/wd/hub/\n\n```elixir\nconfig :wallaby,\n  selenium: [\n    remote_url: \"http://selenium_url\"\n  ]\n```","ref":"Wallaby.Selenium.html#module-selenium-remote-url"},{"type":"module","title":"Default Capabilities - Wallaby.Selenium","doc":"By default, Selenium will use the following capabilities\n\nYou can read more about capabilities in the [JSON Wire Protocol](https://github.com/SeleniumHQ/selenium/wiki/JsonWireProtocol#capabilities-json-object) documentation.\n\n```elixir\n%{\n  javascriptEnabled: true,\n  browserName: \"firefox\",\n  \"moz:firefoxOptions\": %{\n    args: [\"-headless\"]\n  }\n}\n```","ref":"Wallaby.Selenium.html#module-default-capabilities"},{"type":"module","title":"Notes - Wallaby.Selenium","doc":"- Requires [selenium-server](https://www.seleniumhq.org/download/) to be running on port 4444. Wallaby does _not_ manage the start/stop of the Selenium server.\n- Requires [GeckoDriver](https://github.com/mozilla/geckodriver) to be installed in your path when using [Firefox](https://www.mozilla.org/en-US/firefox/new/). Firefox is used by default.","ref":"Wallaby.Selenium.html#module-notes"},{"type":"function","title":"Wallaby.Selenium.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Wallaby.Selenium.html#child_spec/1"},{"type":"function","title":"Wallaby.Selenium.element_location/1","doc":"","ref":"Wallaby.Selenium.html#element_location/1"},{"type":"function","title":"Wallaby.Selenium.element_size/1","doc":"","ref":"Wallaby.Selenium.html#element_size/1"},{"type":"function","title":"Wallaby.Selenium.send_keys/2","doc":"Simulates typing into an element.\n\nWhen sending keys to an element and `keys` is identified as\na local file, the local file is uploaded to the\nSelenium server, returning a file path which is then\nset to the file input we are interacting with.\n\nWe then call the `WebdriverClient.send_keys/2` to set the\nremote file path as the input's value.","ref":"Wallaby.Selenium.html#send_keys/2"},{"type":"type","title":"Wallaby.Selenium.start_session_opts/0","doc":"Options to pass to Wallaby.start_session/1\n\n```elixir\nWallaby.start_session(\n  remote_url: \"http://selenium_url\",\n  capabilities: %{browserName: \"firefox\"}\n)\n```","ref":"Wallaby.Selenium.html#t:start_session_opts/0"},{"type":"module","title":"Wallaby.Session","doc":"Struct containing details about the webdriver session.","ref":"Wallaby.Session.html"},{"type":"type","title":"Wallaby.Session.t/0","doc":"","ref":"Wallaby.Session.html#t:t/0"},{"type":"exception","title":"Wallaby.BadMetadataError","doc":"","ref":"Wallaby.BadMetadataError.html"},{"type":"exception","title":"Wallaby.CookieError","doc":"","ref":"Wallaby.CookieError.html"},{"type":"exception","title":"Wallaby.DependencyError","doc":"","ref":"Wallaby.DependencyError.html"},{"type":"type","title":"Wallaby.DependencyError.t/0","doc":"","ref":"Wallaby.DependencyError.html#t:t/0"},{"type":"exception","title":"Wallaby.ExpectationNotMetError","doc":"","ref":"Wallaby.ExpectationNotMetError.html"},{"type":"exception","title":"Wallaby.JSError","doc":"","ref":"Wallaby.JSError.html"},{"type":"exception","title":"Wallaby.NoBaseUrlError","doc":"","ref":"Wallaby.NoBaseUrlError.html"},{"type":"exception","title":"Wallaby.QueryError","doc":"","ref":"Wallaby.QueryError.html"},{"type":"exception","title":"Wallaby.StaleReferenceError","doc":"","ref":"Wallaby.StaleReferenceError.html"},{"type":"extras","title":"Introduction","doc":"# ![Wallaby](https://i.imgur.com/eQ1tlI3.png)\n\n[![Actions Status](https://github.com/elixir-wallaby/wallaby/workflows/CI/badge.svg)](https://github.com/elixir-wallaby/wallaby/actions)\n[![Module Version](https://img.shields.io/hexpm/v/wallaby.svg)](https://hex.pm/packages/wallaby)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/wallaby/)\n[![License](https://img.shields.io/hexpm/l/wallaby.svg)](https://github.com/elixir-wallaby/wallaby/blob/master/LICENSE)\n\nWallaby helps you test your web applications by simulating realistic user interactions.\nBy default it runs each test case concurrently and manages browsers for you.\nHere's an example test for a simple Todo application:\n\n```elixir\ndefmodule MyApp.Features.TodoTest do\n  use ExUnit.Case, async: true\n  use Wallaby.Feature\n\n  import Wallaby.Query, only: [css: 2, text_field: 1, button: 1]\n\n  feature \"users can create todos\", %{session: session} do\n    session\n    |> visit(\"/todos\")\n    |> fill_in(text_field(\"New Todo\"), with: \"Write my first Wallaby test\")\n    |> click(button(\"Save\"))\n    |> assert_has(css(\".alert\", text: \"You created a todo\"))\n    |> assert_has(css(\".todo-list > .todo\", text: \"Write my first Wallaby test\"))\n  end\nend\n```\n\nBecause Wallaby manages multiple browsers for you, its possible to test several users interacting with a page simultaneously.\n\n```elixir\ndefmodule MyApp.Features.MultipleUsersTest do\n  use ExUnit.Case, async: true\n  use Wallaby.Feature\n\n  import Wallaby.Query, only: [text_field: 1, button: 1, css: 2]\n\n  @page message_path(Endpoint, :index)\n  @message_field text_field(\"Share Message\")\n  @share_button button(\"Share\")\n\n  def message(msg), do: css(\".messages > .message\", text: msg)\n\n  @sessions 2\n  feature \"That users can send messages to each other\", %{sessions: [user1, user2]} do\n    user1\n    |> visit(@page)\n    |> fill_in(@message_field, with: \"Hello there!\")\n    |> click(@share_button)\n\n    user2\n    |> visit(@page)\n    |> fill_in(@message_field, with: \"Hello yourself\")\n    |> click(@share_button)\n\n    user1\n    |> assert_has(message(\"Hello yourself\"))\n\n    user2\n    |> assert_has(message(\"Hello there!\"))\n  end\nend\n```\n\nRead on to see what else Wallaby can do or check out the [Official Documentation](https://hexdocs.pm/wallaby).","ref":"readme.html"},{"type":"extras","title":"Sponsors - Introduction","doc":"_Your company's name and logo could be here!_","ref":"readme.html#sponsors"},{"type":"extras","title":"Setup - Introduction","doc":"","ref":"readme.html#setup"},{"type":"extras","title":"Requirements - Introduction","doc":"Wallaby requires Elixir 1.12+ and OTP 22+.\n\nWallaby also requires `bash` to be installed. Generally `bash` is widely available, but it does not come pre-installed on Alpine Linux.","ref":"readme.html#requirements"},{"type":"extras","title":"Installation - Introduction","doc":"Add Wallaby to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:wallaby, \"~> 0.30\", runtime: false, only: :test}\n  ]\nend\n```\n\nConfigure the driver.\n\n```elixir\n# Chrome\nconfig :wallaby, driver: Wallaby.Chrome # default\n\n# Selenium\nconfig :wallaby, driver: Wallaby.Selenium\n```\n\nYou'll need to install the actual drivers as well.\n\n- Chrome\n  - [`chromedriver`](https://chromedriver.chromium.org/downloads)\n\n- Selenium\n  - [`selenium`](https://www.selenium.dev/downloads/)\n  - [`geckodriver`](https://github.com/mozilla/geckodriver) (for Firefox) or [`chromedriver`](https://chromedriver.chromium.org/downloads) (for Chrome)\n\nEnsure that Wallaby is started in your `test_helper.exs`:\n\n```elixir\n{:ok, _} = Application.ensure_all_started(:wallaby)\n```\n\nWhen calling `use Wallaby.Feature` and using Ecto, please configure your `otp_app`.\n\n```elixir\nconfig :wallaby, otp_app: :your_app\n```","ref":"readme.html#installation"},{"type":"extras","title":"Phoenix - Introduction","doc":"Enable Phoenix to serve endpoints in tests:\n\n```elixir\n# config/test.exs\n\nconfig :your_app, YourAppWeb.Endpoint,\n  server: true\n```\n\nIn your `test_helper.exs` you can provide some configuration to Wallaby.\nAt a minimum, you need to specify a `:base_url`, so Wallaby knows how to resolve relative paths.\n\n```elixir\n# test/test_helper.exs\n\nApplication.put_env(:wallaby, :base_url, YourAppWeb.Endpoint.url)\n```\n\n#### Ecto\n\nIf you're testing a Phoenix application with Ecto and a database that [supports sandbox mode](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html), you can enable concurrent testing by adding the `Phoenix.Ecto.SQL.Sandbox` plug to your `Endpoint`.\nIt's important that this is at the top of `endpoint.ex` before any other plugs.\n\n```elixir\n# lib/your_app_web/endpoint.ex\n\ndefmodule YourAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :your_app\n\n  if sandbox = Application.compile_env(:your_app, :sandbox, false) do\n    plug Phoenix.Ecto.SQL.Sandbox, sandbox: sandbox\n  end\n\n  # ...\n\n  socket(\"/live\", Phoenix.LiveView.Socket,\n    websocket: [connect_info: [:user_agent, session: @session_options]]\n  )\n```\n\nIt's also important to make sure the `user_agent` is passed in the `connect_info` in order to allow the database and browser session to be wired up correctly.\n\nThen make sure sandbox is enabled:\n\n```elixir\n# config/test.exs\n\nconfig :your_app, :sandbox, Ecto.Adapters.SQL.Sandbox\n```\n\nThis enables the database connection to be owned by the process that is running your test, but the connection is shared to the process receiving the HTTP requests from the browser, so that the same data is visible in both processes.\n\nIf you have other resources that should be shared by both processes (e.g. expectations or stubs if using [Mox](https://hexdocs.pm/mox/Mox.html)), then you can define a custom sandbox module:\n\n```elixir\n# test/support/sandbox.ex\n\ndefmodule YourApp.Sandbox do\n  def allow(repo, owner_pid, child_pid) do\n    # Delegate to the Ecto sandbox\n    Ecto.Adapters.SQL.Sandbox.allow(repo, owner_pid, child_pid)\n\n    # Add custom process-sharing configuration\n    Mox.allow(MyMock, owner_pid, child_pid)\n  end\nend\n```\n\nAnd update the test config to use your custom sandbox:\n\n```elixir\n# config/test.exs\n\nconfig :your_app, :sandbox, YourApp.Sandbox\n```\n\n#### Assets\n\nAssets are not re-compiled when you run `mix test`.\nThis can lead to confusion if you've made changes in JavaScript or CSS but tests are still failing.\nThere are two common ways to avoid this confusion.\n\n##### esbuild\n\nIf you're using [`esbuild`](https://hex.pm/packages/esbuild) you can add `esbuild default` to the `test` alias in the mix config file.\n\n```elixir\n  defp aliases do\n    [\n      \"test\": [\n        \"esbuild default\",\n        \"ecto.create --quiet\",\n        \"ecto.migrate\",\n        \"test\",\n      ]\n    ]\n  end\n```\n\n##### Webpack\n\nThe first solution is to run `webpack --mode development --watch` from the assets directory.\nThis will ensure that assets get recompiled after any changes.\n\nThe second solution is to add a new alias to your mix config that recompiles assets for you:\n\n```elixir\n  def project do\n    [\n      app: :my_app,\n      version: \"1.0.0\",\n      aliases: aliases()\n    ]\n  end\n\n  defp aliases, do: [\n    \"test\": [\n      \"assets.compile --quiet\",\n      \"ecto.create --quiet\",\n      \"ecto.migrate\",\n      \"test\",\n    ],\n    \"assets.compile\": &compile_assets/1\n  ]\n\n  defp compile_assets(_) do\n    Mix.shell().cmd(\"cd assets && ./node_modules/.bin/webpack --mode development\",\n      quiet: true\n    )\n  end\n```\n\nThis method is less error prone but it will cause a delay when starting your test suite.\n\n#### LiveView\n\nIn order to test Phoenix LiveView (as of [version 0.17.7](https://github.com/phoenixframework/phoenix_live_view/releases/tag/v0.17.7)) with Wallaby you'll also need to add a function to each `mount/3` function in your LiveViews, or use the `on_mount` `live_session` lifecycle hook in the router:\n\n```elixir\ndefmodule MyApp.Hooks.AllowEctoSandbox do\n  import Phoenix.LiveView\n  import Phoenix.Component\n\n  def on_mount(:default, _params, _session, socket) do\n    allow_ecto_sandbox(socket)\n    {:cont, socket}\n  end\n\n  defp allow_ecto_sandbox(socket) do\n    %{assigns: %{phoenix_ecto_sandbox: metadata}} =\n      assign_new(socket, :phoenix_ecto_sandbox, fn ->\n        if connected?(socket), do: get_connect_info(socket, :user_agent)\n      end)\n\n    Phoenix.Ecto.SQL.Sandbox.allow(metadata, Application.get_env(:your_app, :sandbox))\n  end\nend\n```\n\nand then including the function usage in the router:\n\n```elixir\nlive_session :default, on_mount: MyApp.Hooks.AllowEctoSandbox do\n  # ...\nend\n```\n\n#### Umbrella Apps\n\nIf you're testing an umbrella application containing a Phoenix application for the web interface (`MyWebApp`) and a separate persistence application (`MyPersistenceApp`) using Ecto 2 or 3 with a database that supports sandbox mode, then you can use the same setup as above, with a few tweaks.\n\n```elixir\n# my_web_app/lib/endpoint.ex\n\ndefmodule MyWebApp.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_web_app\n\n  if Application.get_env(:my_persistence_app, :sql_sandbox) do\n    plug Phoenix.Ecto.SQL.Sandbox\n  end\n```\n\nMake sure `MyWebApp` is set up to serve endpoints in tests and that the SQL sandbox is enabled:\n\n```elixir\n# my_web_app/config/test.exs\n\nconfig :my_web_app, MyWebApp.Endpoint,\n  server: true\n\nconfig :my_persistence_app, :sql_sandbox, true\n```\n\nThen in `MyWebApp`'s `test_helper.exs` you can provide some configuration to Wallaby.\nAt minimum, you need to specify a `:base_url`, so Wallaby knows how to resolve relative paths.\n\n```elixir\n# my_web_app/test/test_helper.exs\n\nApplication.put_env(:wallaby, :base_url, MyWebApp.Endpoint.url)\n```\n\nYou will also want to add `phoenix_ecto` as a dependency to `MyWebApp`:\n\n```elixir\n# my_web_app/mix.exs\n\ndef deps do\n  [\n    {:phoenix_ecto, \"~> 3.0\", only: :test}\n  ]\nend\n```","ref":"readme.html#phoenix"},{"type":"extras","title":"Writing tests - Introduction","doc":"It's easiest to add Wallaby to your test suite by using the `Wallaby.Feature` module.\n\n```elixir\ndefmodule YourApp.UserListTest do\n  use ExUnit.Case, async: true\n  use Wallaby.Feature\n\n  feature \"users have names\", %{session: session} do\n    session\n    |> visit(\"/users\")\n    |> find(Query.css(\".user\", count: 3))\n    |> List.first()\n    |> assert_has(Query.css(\".user-name\", text: \"Chris\"))\n  end\nend\n```","ref":"readme.html#writing-tests"},{"type":"extras","title":"API - Introduction","doc":"The full documentation for the DSL is in the [official documentation](https://hexdocs.pm/wallaby).","ref":"readme.html#api"},{"type":"extras","title":"Queries and Actions - Introduction","doc":"Wallaby's API is broken into 2 concepts: Queries and Actions.\n\nQueries allow us to declaratively describe the elements that we would like to interact with and Actions allow us to use those queries to interact with the DOM.\n\nLets say that our html looks like this:\n\n```html\n \n   \n     Ada \n   \n   \n     Grace \n   \n   \n     Alan \n   \n \n```\n\nIf we wanted to interact with all of the users then we could write a query like so `css(\".user\", count: 3)`.\n\nIf we only wanted to interact with a specific user then we could write a query like this `css(\".user-name\", count: 1, text: \"Ada\")`. Now we can use those queries with some actions:\n\n```elixir\nsession\n|> find(css(\".user\", count: 3))\n|> List.first\n|> assert_has(css(\".user-name\", count: 1, text: \"Ada\"))\n```\n\nThere are several queries for common html elements defined in the [Query module](https://hexdocs.pm/wallaby/Wallaby.Query.html#content).\nAll actions accept a query.\nThis makes it easy to use queries we've already defined.\nActions will block until the query is either satisfied or the action times out.\nBlocking reduces race conditions when elements are added or removed dynamically.","ref":"readme.html#queries-and-actions"},{"type":"extras","title":"Navigation - Introduction","doc":"We can navigate directly to pages with `visit`:\n\n```elixir\nvisit(session, \"/page.html\")\nvisit(session, user_path(Endpoint, :index, 17))\n```\n\nIt's also possible to click links directly:\n\n```elixir\nclick(session, link(\"Page 1\"))\n```","ref":"readme.html#navigation"},{"type":"extras","title":"Finding - Introduction","doc":"We can find a specific element or list of elements with `find`:\n\n```elixir\n@user_form   css(\".user-form\")\n@name_field  text_field(\"Name\")\n@email_field text_field(\"Email\")\n@save_button button(\"Save\")\n\nfind(page, @user_form, fn(form) ->\n  form\n  |> fill_in(@name_field, with: \"Chris\")\n  |> fill_in(@email_field, with: \"c@keathley.io\")\n  |> click(@save_button)\nend)\n```\n\nPassing a callback to `find` will return the parent which makes it easy to chain `find` with other actions:\n\n```elixir\npage\n|> find(css(\".users\"), & assert has?(&1, css(\".user\", count: 3)))\n|> click(link(\"Next Page\"))\n```\n\nWithout the callback `find` returns the element.\nThis provides a way to scope all future actions within an element.\n\n```elixir\npage\n|> find(css(\".user-form\"))\n|> fill_in(text_field(\"Name\"), with: \"Chris\")\n|> fill_in(text_field(\"Email\"), with: \"c@keathley.io\")\n|> click(button(\"Save\"))\n```","ref":"readme.html#finding"},{"type":"extras","title":"Interacting with forms - Introduction","doc":"There are a few ways to interact with form elements on a page:\n\n```elixir\nfill_in(session, text_field(\"First Name\"), with: \"Chris\")\nclear(session, text_field(\"last_name\"))\nclick(session, option(\"Some option\"))\nclick(session, radio_button(\"My Fancy Radio Button\"))\nclick(session, button(\"Some Button\"))\n```\n\nIf you need to send specific keys to an element, you can do that with `send_keys`:\n\n```elixir\nsend_keys(session, [\"Example\", \"Text\", :enter])\n```","ref":"readme.html#interacting-with-forms"},{"type":"extras","title":"Assertions - Introduction","doc":"Wallaby provides custom assertions to make writing tests easier:\n\n```elixir\nassert_has(session, css(\".signup-form\"))\nrefute_has(session, css(\".alert\"))\nhas?(session, css(\".user-edit-modal\", visible: false))\n```\n\n`assert_has` and `refute_has` both take a parent element as their first argument.\nThey return that parent, making it easy to chain them together with other actions.\n\n```elixir\nsession\n|> assert_has(css(\".signup-form\"))\n|> fill_in(text_field(\"Email\", with: \"c@keathley.io\"))\n|> click(button(\"Sign up\"))\n|> refute_has(css(\".error\"))\n|> assert_has(css(\".alert\", text: \"Welcome!\"))\n```","ref":"readme.html#assertions"},{"type":"extras","title":"Window Size - Introduction","doc":"You can set the default window size by passing in the `window_size` option into `Wallaby.start_session\\1`.\n\n```elixir\nWallaby.start_session(window_size: [width: 1280, height: 720])\n```\n\nYou can also resize the window and get the current window size during the test.\n\n```elixir\nresize_window(session, 100, 100)\nwindow_size(session)\n```","ref":"readme.html#window-size"},{"type":"extras","title":"Screenshots - Introduction","doc":"It's possible take screenshots:\n\n```elixir\ntake_screenshot(session)\n```\n\nAll screenshots are saved to a `screenshots` directory in the directory that the tests were run in.\n\nIf you want to customize the screenshot directory you can pass it as a config value:\n\n```elixir\n# config/test.exs\nconfig :wallaby, screenshot_dir: \"/file/path\"\n\n# test_helper.exs\nApplication.put_env(:wallaby, :screenshot_dir, \"/file/path\")\n```","ref":"readme.html#screenshots"},{"type":"extras","title":"Automatic screenshots - Introduction","doc":"You can automatically take screenshots on an error when using the `Wallaby.Feature.feature/3` macro.\n\n```elixir\n# config/test.exs\nconfig :wallaby, screenshot_on_failure: true\n\n# test_helper.exs\nApplication.put_env(:wallaby, :screenshot_on_failure, true)\n```","ref":"readme.html#automatic-screenshots"},{"type":"extras","title":"JavaScript - Introduction","doc":"","ref":"readme.html#javascript"},{"type":"extras","title":"Asynchronous code - Introduction","doc":"Testing asynchronous JavaScript code can expose timing issues and race conditions.\nWe might try to interact with an element that hasn't yet appeared on the page.\nElements can become stale while we're trying to interact with them.\n\nWallaby helps solve this by blocking.\nInstead of manually setting timeouts we can use `assert_has` and some declarative queries to block until the DOM is in a good state.\n\n```elixir\nsession\n|> click(button(\"Some Async Button\"))\n|> assert_has(css(\".async-result\"))\n|> click(button(\"Next Action\"))\n```","ref":"readme.html#asynchronous-code"},{"type":"extras","title":"Interacting with dialogs - Introduction","doc":"Wallaby provides several ways to interact with JavaScript dialogs such as `window.alert`, `window.confirm` and `window.prompt`.\n\nYou can use one of the following functions:\n\n- For `window.alert` use `accept_alert/2`\n- For `window.confirm` use `accept_confirm/2` or `dismiss_confirm/2`\n- For `window.prompt` use `accept_prompt/2-3` or `dismiss_prompt/2`\n\nAll of these take a function as last parameter, which must include the necessary interactions to trigger the dialog. For example:\n\n```elixir\nalert_message = accept_alert session, fn(session) ->\n  click(session, link(\"Trigger alert\"))\nend\n```\n\nTo emulate user input for a prompt, `accept_prompt` takes an optional parameter:\n\n```elixir\nprompt_message = accept_prompt session, [with: \"User input\"], fn(session) ->\n  click(session, link(\"Trigger prompt\"))\nend\n```","ref":"readme.html#interacting-with-dialogs"},{"type":"extras","title":"JavaScript logging and errors - Introduction","doc":"Wallaby captures both JavaScript logs and errors.\nAny uncaught exceptions in JavaScript will be re-thrown in Elixir.\nThis can be disabled by specifying `js_errors: false` in your Wallaby config.\n\nJavaScript logs are written to :stdio by default.\nThis can be changed to any IO device by setting the `:js_logger` option in your Wallaby config.\nFor instance if you want to write all JavaScript console logs to a file you could do something like this:\n\n```elixir\n{:ok, file} = File.open(\"browser_logs.log\", [:write])\nApplication.put_env(:wallaby, :js_logger, file)\n```\n\nLogging can be disabled by setting `:js_logger` to `nil`.","ref":"readme.html#javascript-logging-and-errors"},{"type":"extras","title":"Configuration - Introduction","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"Adjusting timeouts - Introduction","doc":"Wallaby uses [hackney](https://github.com/benoitc/hackney) under the hood, so we offer a hook that allows you to control any hackney options you'd like to have sent along on every request.\nThis can be controlled with the `:hackney_options` setting in `config.exs`.\n\n```elixir\nconfig :wallaby,\n  hackney_options: [timeout: :infinity, recv_timeout: :infinity]\n\n# Overriding a value\nconfig :wallaby,\n  hackney_options: [timeout: 5_000]\n```","ref":"readme.html#adjusting-timeouts"},{"type":"extras","title":"Contributing - Introduction","doc":"Wallaby is a community project. Pull Requests (PRs) and reporting issues are greatly welcome.\n\nTo get started and setup the project, make sure you've got Elixir 1.7+ installed and then:","ref":"readme.html#contributing"},{"type":"extras","title":"Development Dependencies - Introduction","doc":"Wallaby requires the following tools.\n\n- ChromeDriver\n- Google Chrome\n- GeckoDriver\n- Mozilla Firefox\n- selenium-server-standalone\n\n```shell\n# Unit tests\n$ mix test\n\n# Integration tests for all drivers\n$ mix test.drivers\n\n# Integration tests for a specific driver\n$ WALLABY_DRIVER=chrome mix test\n$ WALLABY_DRIVER=selenium mix test\n\n# All tests\n$ mix test.all\n```","ref":"readme.html#development-dependencies"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}